import React from 'react';
import {List, ListItem, ListItemIcon, Tooltip, ListItemText, Divider} from '@material-ui/core';
import {ArrowBack as ArrowBackIcon, Settings as SettingIcon, 
    Collections as CollectionIcon, VideoLibrary as VideoIcon, Home as HomeIcon,
    Folder as FolderIcon } from '@material-ui/icons';
import {Link as RouterLink, useHistory} from 'react-router-dom';

export const NavItem = (props:{name:string,to:string, icon:React.ReactElement<any,any>})=>{
    const history = useHistory();
    return (<ListItem button key={props.name} component={RouterLink} to={props.to}>
    <ListItemIcon>
        <Tooltip title={props.name.toLocaleLowerCase()} placement="bottom">
            {props.icon}
        </Tooltip>
    </ListItemIcon>
    <ListItemText primary={props.name}></ListItemText>
</ListItem>);
}

export const NavList = (props: {children?:React.ReactNode})=>{
    return (<List>
            {props.children}
        </List>);
}

export const BackItem = (props:{to?:string})=>{
    return <NavItem name="Back" to={props.to ?? "/"} icon={<ArrowBackIcon></ArrowBackIcon>}/>;
}

export function CommonMenuList(props?:{url?:string}) {
    let url = props?.url || "";
    return (<NavList>
        {url !== "" && <><BackItem /> <Divider /></>}
        <NavItem name="All" to="/" icon={<HomeIcon />} />
        <NavItem name="Manga" to="/search?content_type=manga" icon={<CollectionIcon />}></NavItem>
        <NavItem name="Video" to="/search?content_type=video" icon={<VideoIcon />} />
        <Divider />
        <NavItem name="Difference" to="/difference" icon={<FolderIcon/>}></NavItem>
        <NavItem name="Settings" to="/setting" icon={<SettingIcon />} />
    </NavList>);
}