import React from 'react';
import {ChipTypeMap} from '@material-ui/core/Chip';
import { Chip, colors } from '@material-ui/core';
import {useTheme, makeStyles, Theme, emphasize, fade} from '@material-ui/core/styles';
import {Link as RouterLink} from 'react-router-dom';

type TagChipStyleProp = {
    color: string
}

const useTagStyles = makeStyles((theme:Theme)=>({
    root:(props:TagChipStyleProp)=>({
        color: theme.palette.getContrastText(props.color),
        backgroundColor: props.color,
    }),
    clickable:(props:TagChipStyleProp)=>({
        '&:hover, &:focus':{
            backgroundColor:emphasize(props.color,0.08)
        }
    }),
    deletable: {
        '&:focus': {
            backgroundColor: (props:TagChipStyleProp)=>emphasize(props.color, 0.2),
        }
    },
    outlined:{
        color: (props:TagChipStyleProp)=>props.color,
        border: (props:TagChipStyleProp)=> `1px solid ${props.color}`,
        '$clickable&:hover, $clickable&:focus, $deletable&:focus': {
            backgroundColor:(props:TagChipStyleProp)=>fade(props.color,theme.palette.action.hoverOpacity),
        },
    },
    icon:{
        color:"inherit",
    },
    deleteIcon:{
        color:(props:TagChipStyleProp)=>fade(theme.palette.getContrastText(props.color),0.7),
        "&:hover, &:active":{
            color:(props:TagChipStyleProp)=>theme.palette.getContrastText(props.color),
        }
    }
}));

const {blue, pink} = colors;
const getTagColorName = (tagname :string):string=>{
    if(tagname.startsWith("female")){
        return pink[600];
    }
    else if(tagname.startsWith("male")){
        return blue[600];
    }
    else return "default";
}

type ColorChipProp = Omit<ChipTypeMap['props'],"color"> & TagChipStyleProp & {
    component?: React.ElementType,
    to?: string
}

export const ColorChip = (props:ColorChipProp)=>{
    const {color,...rest} = props;
    const classes = useTagStyles({color : color !== "default" ? color : "#000"});
    return <Chip color="default" classes={{
        ...(color !== "default" ? classes : {})
    }} {...rest}></Chip>;
}

type TagChipProp = Omit<ChipTypeMap['props'],"color"> & {
    tagname:string
}

export const TagChip = (props:TagChipProp)=>{
    const {tagname,label,clickable,...rest} = props;
    let newlabel:string|undefined = undefined;
    if(typeof label === "string"){
        if(label.startsWith("female:")){
            newlabel ="♀ "+label.slice(7);
        }
        else if(label.startsWith("male:")){
            newlabel = "♂ "+label.slice(5);
        }
    }
    const inner = clickable ? 
    (<ColorChip color={getTagColorName(tagname)} clickable={clickable} label={newlabel||label} {...rest} 
    component={RouterLink} to={`/search?allow_tag=${tagname}`}></ColorChip>):
    (<ColorChip color={getTagColorName(tagname)} clickable={clickable} label={newlabel||label} {...rest}></ColorChip>);
    return inner;
}