import {basename, dirname, join as pathjoin} from 'path';
import {Document, DocumentAccessor} from '../model/mod';
import { ContentFile, createContentFile } from '../content/mod';
import {IDiffWatcher} from './watcher';
import {ContentList} from './content_list';

//refactoring needed.
export class ContentDiffHandler{
    waiting_list:ContentList;
    tombstone: Map<string,Document>;//hash, contentfile
    doc_cntr: DocumentAccessor;
    content_type: string;
    constructor(cntr: DocumentAccessor,content_type:string){
        this.waiting_list = new ContentList();
        this.tombstone = new Map<string,Document>();
        this.doc_cntr = cntr;
        this.content_type = content_type;
    }
    async setup(){
        const deleted = await this.doc_cntr.findDeleted(this.content_type);
        for (const it of deleted) {
            this.tombstone.set(it.content_hash,it);
        }
    }
    register(diff:IDiffWatcher){
        diff.on('create',(path)=>this.OnCreated(path))
            .on('delete',(path)=>this.OnDeleted(path))
            .on('change',(prev,cur)=>this.OnChanged(prev,cur));
    }
    private async OnDeleted(cpath: string){
        const basepath = dirname(cpath);
        const filename = basename(cpath);
        if(this.waiting_list.hasPath(cpath)){
            this.waiting_list.deletePath(cpath);
            return;
        }
        const dbc = await this.doc_cntr.findByPath(basepath,filename);
        if(dbc.length === 0) return; //ignore
        if(this.waiting_list.hasHash(dbc[0].content_hash)){
            //if path changed, update changed path.
            await this.doc_cntr.update({
                id:dbc[0].id,
                deleted_at: null,
                filename:filename,
                basepath:basepath
            });
            return;
        }
        //db invalidate
        await this.doc_cntr.update({
            id:dbc[0].id,
            deleted_at: Date.now(),
        });
        this.tombstone.set(dbc[0].content_hash, dbc[0]);
    }
    private async OnCreated(cpath:string){
        const basepath = dirname(cpath);
        const filename = basename(cpath);
        const content = createContentFile(this.content_type,pathjoin(basepath,filename));
        const hash = await content.getHash();
        const c = this.tombstone.get(hash);
        if(c !== undefined){
            this.doc_cntr.update({
                id: c.id,
                deleted_at: null,
                filename:filename,
                basepath:basepath
            });
            return;
        }
        this.waiting_list.set(content);
    }
    private async OnChanged(prev_path:string,cur_path:string){
        const prev_basepath = dirname(prev_path);
        const prev_filename = basename(prev_path);
        const cur_basepath = dirname(cur_path);
        const cur_filename = basename(cur_path);
        const doc = await this.doc_cntr.findByPath(prev_basepath,prev_filename);
        await this.doc_cntr.update({...doc[0],
            basepath:cur_basepath,
            filename:cur_filename});
    }
}