import { ContentFile } from '../content/mod';
import event from 'events';

interface ContentListEvent{
    'set':(c:ContentFile)=>void,
    'delete':(c:ContentFile)=>void,
}

export class ContentList extends event.EventEmitter{
    cl:Map<string,ContentFile>;
    hl:Map<string,ContentFile>;
    on<U extends keyof ContentListEvent>(event:U,listener:ContentListEvent[U]): this{
        return super.on(event,listener);
    }
    emit<U extends keyof ContentListEvent>(event:U,...arg:Parameters<ContentListEvent[U]>): boolean{
        return super.emit(event,...arg);
    }
    constructor(){
        super();
        this.cl = new Map;
        this.hl = new Map;
    }
    hasHash(s:string){
        return this.hl.has(s);
    }
    hasPath(p:string){
        return this.cl.has(p);
    }
    getHash(s:string){
        return this.hl.get(s)
    }
    getPath(p:string){
        return this.cl.get(p);
    }
    async set(c:ContentFile){
        const path = c.path;
        const hash = await c.getHash();
        this.cl.set(path,c);
        this.hl.set(hash,c);
        this.emit('set',c);
    }
    async delete(c:ContentFile){
        let r = true;
        r = this.cl.delete(c.path) && r;
        r = this.hl.delete(await c.getHash()) && r;
        this.emit('delete',c);
        return r;
    }
    async deletePath(p:string){
        const o = this.getPath(p);
        if(o === undefined) return false;
        return this.delete(o);
    }
    async deleteHash(s:string){
        const o = this.getHash(s);
        if(o === undefined) return false;
        return this.delete(o);
    }
    getAll(){
        return [...this.cl.values()];
    }
}
