import event from 'events';
import {FSWatcher,watch,promises} from 'fs';
import {IDiffWatcher, DiffWatcherEvent} from '../watcher';
import {join} from 'path';
import { DocumentAccessor } from '../../model/doc';
import { setupHelp } from './util';

const {readdir} = promises;

export class CommonDiffWatcher extends event.EventEmitter implements IDiffWatcher{
    on<U extends keyof DiffWatcherEvent>(event:U,listener:DiffWatcherEvent[U]): this{
        return super.on(event,listener);
    }
    emit<U extends keyof DiffWatcherEvent>(event:U,...arg:Parameters<DiffWatcherEvent[U]>): boolean{
        return super.emit(event,...arg);
    }
    private _path:string;
    private _watcher: FSWatcher;

    constructor(path:string){
        super();
        this._path = path;
        this._watcher = watch(this._path,{persistent: true, recursive:false},async (eventType,filename)=>{
            if(eventType === "rename"){
                const cur = await readdir(this._path);
                //add
                if(cur.includes(filename)){
                    this.emit('create',join(this.path,filename));
                }
                else{
                    this.emit('delete',join(this.path,filename))
                }
            }
        });
    }
    async setup(cntr: DocumentAccessor): Promise<void> {
        await setupHelp(this,this.path,cntr);
    }
    public get path(){
        return this._path;
    }
    watchClose(){
        this._watcher.close()
    }
}