import StreamZip, { ZipEntry } from 'node-stream-zip';
import {orderBy} from 'natural-orderby';
import { ReadStream } from 'fs';

export async function readZip(path : string):Promise<StreamZip>{
    return new Promise((resolve,reject)=>{
            let zip = new StreamZip({
                file:path,
                storeEntries: true
            });
            zip.on('error',(err)=>{
                console.error(`read zip file ${path}`);
                reject(err);
            });
            zip.on('ready',()=>{
                resolve(zip);
            });
        }
    );
}
export function entriesByNaturalOrder(zip: StreamZip){
    const entries = zip.entries();
    const ret = orderBy(Object.values(entries),v=>v.name);
    return ret;
}
export async function createReadableStreamFromZip(zip:StreamZip,entry: ZipEntry):Promise<NodeJS.ReadableStream>{
    return new Promise((resolve,reject)=>{
        zip.stream(entry,(err, stream)=>{
            if(stream !== undefined){
                resolve(stream);
            }
            else{
                reject(err);
            }
        });}
    );
}
export async function readAllFromZip(zip:StreamZip,entry: ZipEntry):Promise<Buffer>{
    const stream = await createReadableStreamFromZip(zip,entry);
    const chunks:Uint8Array[] = [];
    return new Promise((resolve,reject)=>{
        stream.on('data',(data)=>{chunks.push(data)});
        stream.on('error', (err)=>reject(err));
        stream.on('end',()=>resolve(Buffer.concat(chunks)));
    });
}